package com.sun.beans.finder;

import java.beans.MethodDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.BeanDescriptor;
import java.lang.reflect.Method;
import java.beans.BeanInfo;

public final class BeanInfoFinder extends InstanceFinder<BeanInfo>
{
    private static final String DEFAULT = "sun.beans.infos";
    private static final String DEFAULT_NEW = "com.sun.beans.infos";
    
    public BeanInfoFinder() {
        super(BeanInfo.class, true, "BeanInfo", new String[] { "sun.beans.infos" });
    }
    
    private static boolean isValid(Class<?> clazz, Method method) {
        return method != null && method.getDeclaringClass().isAssignableFrom(clazz);
    }
    
    @Override
    protected BeanInfo instantiate(Class<?> clazz, String s, String anObject) {
        if ("sun.beans.infos".equals(s)) {
            s = "com.sun.beans.infos";
        }
        BeanInfo beanInfo = (!"com.sun.beans.infos".equals(s) || "ComponentBeanInfo".equals(anObject)) ? ((BeanInfo)super.instantiate(clazz, s, anObject)) : null;
        if (beanInfo != null) {
            BeanDescriptor beanDescriptor = beanInfo.getBeanDescriptor();
            if (beanDescriptor != null) {
                if (clazz.equals(beanDescriptor.getBeanClass())) {
                    return beanInfo;
                }
            }
            else {
                PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
                if (propertyDescriptors != null) {
                    int length2 = propertyDescriptors.length;
                    int j = 0;
                    while (j < length2) {
                        PropertyDescriptor propertyDescriptor = propertyDescriptors[j];
                        Method method = propertyDescriptor.getReadMethod();
                        if (method == null) {
                            method = propertyDescriptor.getWriteMethod();
                        }
                        if (isValid(clazz, method)) {
                            return beanInfo;
                        }
                        ++j;
                    }
                }
                else {
                    MethodDescriptor[] methodDescriptors = beanInfo.getMethodDescriptors();
                    if (methodDescriptors != null) {
                        int length = methodDescriptors.length;
                        int i = 0;
                        while (i < length) {
                            if (isValid(clazz, methodDescriptors[i].getMethod())) {
                                return beanInfo;
                            }
                            ++i;
                        }
                    }
                }
            }
        }
        return null;
    }
}
